/**
 * All-in-one start script, starts everything you need to try N|Solid Console.
 *
 * 1. Start service dependencies e.g. etcd, nsolid-proxy
 * 2. Start the collector (starts an HTTP server)
 * 3. Simulate some Applications with real Processes
 *
 */

'use strict'

require('./polyfill')

const spawn = require('child_process').spawn
const which = require('npm-which')(process.cwd())
const config = require('../src/collect/config')
const utils = require('./utils')
const stderrNamespace = utils.stderrNamespace

const argv = config(process.argv, `
Boots everything you need to try N|Solid Console.

Usage: $0 [options]`.trim())

process
.on('uncaughtException', onError)
.on('unhandledRejection', onError)

function onError (err) {
  if (err) throw err
}

var consoleEnv = Object.assign({}, process.env)

if (argv['include-nsolid-console']) {
  Object.assign(consoleEnv, {
    NSOLID_APPNAME: 'N|Solid',
    NSOLID_SOCKET: process.env.NSOLID_SOCKET || '127.0.0.1:0',
    NSOLID_HUB: '127.0.0.1:4001'
  })
}

stderrNamespace('etcd', spawn('etcd', ['--cors=*'], {}))

var pathToProxy = which.sync('nsolid-proxy')

stderrNamespace('nsolid-proxy', spawn(pathToProxy, {
  cwd: process.cwd(),
  env: consoleEnv
}), {invertStdio: true})

stderrNamespace('nsolid-console', spawn('node', [
  'bin/nsolid-console.js',
  '--port', argv.port,
  '--interval', 1000
].concat(process.argv.slice(2)), {
  env: Object.assign({}, process.env, consoleEnv, {
    NODE_ENV: process.env.NODE_ENV || 'development',
    DEBUG: process.env.DEBUG + ',-engine*'
  })
}))

// spawn demo processes
require('./demo-processes')
