const test = require('tape')
const tmp = require('tmp')
const path = require('path')
const fs = require('fs')
const mkdirp = require('mkdirp')
const through = require('through')
const createProfile = require('../profile')
const config = require('../../config')
const createTestCleanup = require('../../test/_cleanup')
const createNetwork = require('../../test/_create-network')
const createNSolidAgent = require('../../test/_nsolid-agent')
const Cache = require('../../cache')

tmp.setGracefulCleanup()

test('actions/profile - create profile', createTestCleanup(t => {
  const app = 'profile-generators'
  const id = 'the-one-we-care-about'
  const tmpobj = tmp.dirSync({ unsafeCleanup: true })
  const profilePath = path.join(tmpobj.name, 'profiles')
  mkdirp.sync(profilePath)

  createNetwork(t, app, 1)(function (e, network) {
    t.error(e, 'created network')
    const conf = config(['--hub', network.proxyUrl])
    const cache = new Cache(tmpobj.name)

    createNSolidAgent(t, network, app, id, (agentError, listener) => {
      t.error(agentError, 'created agent')

      listener.on('profile_start', (req, nop, next) => {
        next(null, {
          started: Date.now(),
          collecting: true
        })
        return true
      })

      const expect = {
        profile: true
      }

      listener.on('profile_stop', (req, nop, next) => {
        const stream = through()
        next(null, stream)

        process.nextTick(() => {
          stream.end(JSON.stringify(expect))
        })

        return true
      })

      const proc = {
        app: app,
        id: id,
        hub: network.proxyUrl,
        title: 'Mr Profiles',
        pid: 123
      }

      cache.addProcess(proc)

      createProfile(cache, conf, proc, 100, (e, meta) => {
        t.equal(meta.pid, 123)
        t.equal(meta.title, 'Mr Profiles')

        fs.readFile(path.join(profilePath, meta.file), { encoding: 'utf8' }, (e, str) => {
          const obj = JSON.parse(str)
          t.deepEqual(obj, expect, 'loaded from disk')
          t.end()
        })
      })
    })
  })
}))
