const test = require('tape')
const tmp = require('tmp')
const path = require('path')
const fs = require('fs')
const mkdirp = require('mkdirp')
const through = require('through')
const createSnapshot = require('../snapshot')
const config = require('../../config')
const createTestCleanup = require('../../test/_cleanup')
const createNetwork = require('../../test/_create-network')
const createNSolidAgent = require('../../test/_nsolid-agent')
const Cache = require('../../cache')

tmp.setGracefulCleanup()

test('actions/snapshot - create snapshot', createTestCleanup(t => {
  const app = 'snapshot-generators'
  const id = 'the-one-we-care-about'
  const tmpobj = tmp.dirSync({ unsafeCleanup: true })
  const snapshotPath = path.join(tmpobj.name, 'snapshots')
  mkdirp.sync(snapshotPath)

  createNetwork(t, app, 1)(function (e, network) {
    t.error(e, 'created network')
    const conf = config(['--hub', network.proxyUrl])
    const cache = new Cache(tmpobj.name)

    createNSolidAgent(t, network, app, id, (agentError, listener) => {
      t.error(agentError, 'created agent')

      const expect = {
        snapshot: true
      }

      listener.on('snapshot', (req, nop, next) => {
        const stream = through()
        next(null, stream)

        process.nextTick(() => {
          stream.end(JSON.stringify(expect))
        })

        return true
      })

      const proc = {
        app: app,
        id: id,
        hub: network.proxyUrl,
        title: 'Mr Snapshots',
        pid: 123
      }

      cache.addProcess(proc)

      createSnapshot(cache, conf, proc, (e, meta) => {
        t.error(e, 'created snapshot')
        t.equal(meta.pid, 123, 'pid matches')
        t.equal(meta.title, 'Mr Snapshots', 'title matches')

        fs.readFile(path.join(snapshotPath, meta.file), { encoding: 'utf8' }, (e, str) => {
          const obj = JSON.parse(str)
          t.deepEqual(obj, expect, 'loaded from disk')
          t.end()
        })
      })
    })
  })
}))
