'use strict'

const createGraph = require('../graph-cache')
const computeAppId = require('../../compute-app-id')
const camelize = require('camelize')

module.exports = computeSparklines

function toFixed (v) {
  return Math.round(v * 100) / 100
}

function computeSparklines (config, application, hosts, duration) {
  var storeTime = (duration * 1000 * 60)

  const rc = config.rc
  const appid = computeAppId(application.name)

  if (rc) {
    const thresholds = config.rc.thresholds

    if (thresholds && thresholds.app && thresholds.app[appid]) {
      const appThresholds = thresholds.app[appid]
      const appGraph = createGraph({type: 'app', id: appid}, storeTime)

      Object.keys(appThresholds).forEach((metricName) => {
        const thresholdConfig = appThresholds[metricName]
        if (!thresholdConfig) {
          return
        }

        let threshold = thresholdConfig.threshold
        if (!thresholdConfig.enabled) {
          threshold = null
        }

        // camelize threshold-cpu to thresholdCpu for cleaner access on the frontend
        const graphName = camelize('threshold-' + metricName)

        appGraph.add(graphName, threshold)
      })
    }
  }

  application.processes.forEach(function eachProcess (proc) {
    var hostname = proc.host
    var host = hosts[hostname]
    var processGraph = createGraph({type: 'process', id: proc.id}, storeTime)
    var hostGraph = createGraph({type: 'host', id: hostname}, storeTime)

    if (!host.computedSparklines) {
      host.computedSparklines = true
      hostGraph.add('load1m', toFixed(host.stats.load1m))
      hostGraph.add('memoryPercentageUsed', toFixed(host.stats.memoryPercentageUsed))
    }

    var hostData = hostGraph.get('load1m')

    processGraph.add('cpu', toFixed(proc.cpu), hostData)
    processGraph.add('rss', proc.memory.rss, hostData)
    processGraph.add('heapTotal', proc.memory.total, hostData)
    processGraph.add('heapUsed', proc.memory.used, hostData)
    processGraph.add('asyncHandles', proc.async.handles, hostData)
    processGraph.add('asyncRequests', proc.async.requests, hostData)
  })
}
