var times = [
  { ms: 31557600000, abbrev: 'y' },
  { ms: 2629800000, abbrev: 'mo' },
  { ms: 604800000, abbrev: 'w' },
  { ms: 86400000, abbrev: 'd' },
  { ms: 3600000, abbrev: 'h' },
  { ms: 60000, abbrev: 'm' },
  { ms: 1000, abbrev: 's' },
  { ms: 1, abbrev: 'ms' }
]

var timeUnit = {
  year: function (unit) { return unit * times[0].ms },
  month: function (unit) { return unit * times[1].ms },
  week: function (unit) { return unit * times[2].ms },
  day: function (unit) { return unit * times[3].ms },
  hour: function (unit) { return unit * times[4].ms },
  minute: function (unit) { return unit * times[5].ms },
  second: function (unit) { return unit * times[6].ms },
  millisecond: function (unit) { return unit * times[7].ms }
}

function hms (timestamp, units) {
  var result = []
  function processTimeUnits (timestamp, counter) {
    if (counter < times.length) {
      var unit = times[counter]
      if (!units || units.indexOf(unit.abbrev) > -1) {
        var count = Math.trunc(timestamp / unit.ms)
        if (count > 0) {
          result.push(count + unit.abbrev)
        }
        var remainder = timestamp - (count * unit.ms)
        if (remainder >= 0) {
          processTimeUnits(remainder, ++counter)
        }
      } else {
        processTimeUnits(timestamp, ++counter)
      }
    }
  }

  processTimeUnits(timestamp, 0)
  return result.join(' ')
}

function ago (timestamp) {
  if (timestamp < timeUnit.second(11)) {
    return 'just now'
  } else if (timestamp <= timeUnit.second(59)) {
    return 'less than a minute ago'
  } else if (timestamp < timeUnit.minute(2)) {
    return 'a minute ago'
  } else if (timestamp < timeUnit.minute(50)) {
    return Math.trunc(timestamp / timeUnit.minute(1)) + ' minutes ago'
  } else if (timestamp <= timeUnit.minute(119)) {
    return 'an hour ago'
  } else if (timestamp < timeUnit.hour(20)) {
    return Math.trunc(timestamp / timeUnit.hour(1)) + ' hours ago'
  } else if (timestamp <= timeUnit.hour(47)) {
    return 'yesterday'
  } else if (timestamp < timeUnit.day(6)) {
    return Math.trunc(timestamp / timeUnit.day(1)) + ' days ago'
  } else if (timestamp <= timeUnit.day(13)) {
    return 'last week'
  } else if (timestamp < timeUnit.week(4)) {
    return Math.trunc(timestamp / timeUnit.week(1)) + ' weeks ago'
  } else if (timestamp < timeUnit.month(2)) {
    return 'last month'
  } else if (timestamp < timeUnit.month(11)) {
    return Math.trunc(timestamp / timeUnit.month(1)) + ' months ago'
  } else if (timestamp < timeUnit.year(2)) {
    return 'last year'
  } else {
    return Math.trunc(timestamp / timeUnit.year(1)) + ' years ago'
  }
}

exports.timeUnit = timeUnit
exports.ago = ago
exports.hms = hms
