var tape = require('tape')
var mte = require('./ms-to-english')
var timestamp = new Date()

tape('abbreviated span', function doTest(t) {
  t.equals(mte.hms(-1000), '')
  t.equals(mte.hms(-1), '')
  t.equals(mte.hms(mte.timeUnit.millisecond(0)), '')
  t.equals(mte.hms(mte.timeUnit.millisecond(1)), '1ms')
  t.equals(mte.hms(mte.timeUnit.millisecond(500)), '500ms')
  t.equals(mte.hms(mte.timeUnit.second(1)), '1s')
  t.equals(mte.hms(mte.timeUnit.second(1) + mte.timeUnit.millisecond(500)), '1s 500ms')
  t.equals(mte.hms(mte.timeUnit.second(2)), '2s')
  t.equals(mte.hms(mte.timeUnit.second(10)), '10s')
  t.equals(mte.hms(mte.timeUnit.minute(1)), '1m')
  t.equals(mte.hms(mte.timeUnit.minute(2)), '2m')
  t.equals(mte.hms(mte.timeUnit.minute(4)), '4m')
  t.equals(mte.hms(mte.timeUnit.minute(7) + mte.timeUnit.second(20)), '7m 20s')
  t.equals(
    mte.hms(mte.timeUnit.hour(1) + mte.timeUnit.minute(30) + mte.timeUnit.second(20)),
    '1h 30m 20s'
  )
  t.equals(
    mte.hms(mte.timeUnit.day(1) + mte.timeUnit.hour(1) + mte.timeUnit.minute(30) + mte.timeUnit.second(20)),
    '1d 1h 30m 20s'
  )
  t.equals(
    mte.hms(mte.timeUnit.year(1) + mte.timeUnit.day(1) + mte.timeUnit.hour(1) + mte.timeUnit.minute(30) + mte.timeUnit.second(20)),
    '1y 1d 1h 30m 20s'
  )
  t.equals(
    mte.hms(mte.timeUnit.year(6) + mte.timeUnit.month(5) + mte.timeUnit.week(1) + mte.timeUnit.day(1) + mte.timeUnit.hour(12) + mte.timeUnit.minute(55) + mte.timeUnit.second(23)),
    '6y 5mo 1w 1d 12h 55m 23s'
  )
  t.equals(
    mte.hms(mte.timeUnit.year(10) + mte.timeUnit.day(1) + mte.timeUnit.hour(1) + mte.timeUnit.minute(30) + mte.timeUnit.second(20)),
    '10y 1d 1h 30m 20s'
  )
  t.equals(
    mte.hms(mte.timeUnit.year(100) + mte.timeUnit.day(1) + mte.timeUnit.hour(1) + mte.timeUnit.minute(30) + mte.timeUnit.second(20)),
    '100y 1d 1h 30m 20s'
  )
  t.equals(
    mte.hms(mte.timeUnit.year(1000000) + mte.timeUnit.day(1) + mte.timeUnit.hour(1) + mte.timeUnit.minute(30) + mte.timeUnit.second(20)),
    '1000000y 1d 1h 30m 20s'
  )
  t.equals(mte.hms(203232323000), '6y 5mo 1w 1d 12h 55m 23s')
  t.equals(mte.hms(218203232323000), '6914y 5mo 1w 2d 16h 28m 43s')
  t.equals(mte.hms(1500, ['s']), '1s')
  t.equals(mte.hms(218203232323250, ['y', 'mo', 'w', 'd', 'h', 'm', 's']), '6914y 5mo 1w 2d 16h 28m 43s')
  t.equals(mte.hms(218203232323250, ['y']), '6914y')
  t.equals(mte.hms(218203232323250, ['y', 'mo']), '6914y 5mo')
  t.equals(mte.hms(218203232323250, ['mo']), '82973mo')
  t.equals(mte.hms(218203232323250, ['w']), '360785w')
  t.equals(mte.hms(218203232323250, ['d']), '2525500d')
  t.equals(mte.hms(218203232323250, ['h']), '60612008h')
  t.equals(mte.hms(218203232323250, ['m']), '3636720538m')
  t.equals(mte.hms(218203232323250, ['s']), '218203232323s')
  t.equals(mte.hms(218203232323250, ['ms']), '218203232323250ms')
  t.equals(mte.hms(218203232323250, ['y', 'ms']), '6914y 13985923250ms')
  t.equals(mte.hms(218203232323250, ['mo', 'm']), '82973mo 13948m')
  t.equals(mte.hms(218203232323250, ['w', 'h']), '360785w 128h')
  t.equals(mte.hms(218203232323250, ['d', 'h']), '2525500d 8h')
  t.end()
})

tape('just now', function doTest (t) {
	t.equals(mte.ago(0), 'just now', '0s')
	t.equals(mte.ago(mte.timeUnit.second(10)), 'just now', '10s')
  t.end()
})

tape('less than a minute ago', function doTest (t) {
  t.equals(mte.ago(mte.timeUnit.second(11)), 'less than a minute ago', '11s')
  t.equals(mte.ago(mte.timeUnit.second(59)), 'less than a minute ago', '59s')
  t.end()
})

tape('a minute ago', function doTest (t) {
	t.equals(mte.ago(mte.timeUnit.minute(1)), 'a minute ago', '60s')
	t.equals(mte.ago(mte.timeUnit.second(70)), 'a minute ago', '70s')
  t.equals(mte.ago(mte.timeUnit.second(119)), 'a minute ago', '119s')
  t.end()
})

tape('minutes', function doTest (t) {
	t.equals(mte.ago(mte.timeUnit.minute(2)), '2 minutes ago', '120s')
	t.equals(mte.ago(mte.timeUnit.minute(20)), '20 minutes ago', '20m')
	t.equals(mte.ago(mte.timeUnit.minute(45)), '45 minutes ago', '45m')
  t.equals(mte.ago(mte.timeUnit.minute(49)), '49 minutes ago', '49m')
  t.end()
})

tape('hour', function doTest (t) {
	t.equals(mte.ago(mte.timeUnit.minute(50)), 'an hour ago', '50m')
	t.equals(mte.ago(mte.timeUnit.minute(60)), 'an hour ago', '60m')
	t.equals(mte.ago(mte.timeUnit.minute(70)), 'an hour ago', '70m')
  t.equals(mte.ago(mte.timeUnit.minute(119)), 'an hour ago', '119m')
  t.end()
})

tape('hours', function doTest (t) {
	t.equals(mte.ago(mte.timeUnit.hour(2)), '2 hours ago', '120m')
	t.equals(mte.ago(mte.timeUnit.hour(3) + mte.timeUnit.minute(30)), '3 hours ago', '210m')
	t.equals(mte.ago(mte.timeUnit.hour(19)), '19 hours ago', '19h')
  t.end()
})

tape('yesterday', function doTest (t) {
	t.equals(mte.ago(mte.timeUnit.hour(20)), 'yesterday', '20h')
	t.equals(mte.ago(mte.timeUnit.hour(40)), 'yesterday', '40h')
	t.equals(mte.ago(mte.timeUnit.hour(47)), 'yesterday', '47h')
  t.end()
})

tape('days', function doTest (t) {
	t.equals(mte.ago(mte.timeUnit.day(2)), '2 days ago', '2d')
	t.equals(mte.ago(mte.timeUnit.day(5)), '5 days ago', '5d')
  t.end()
})

tape('week', function doTest (t) {
	t.equals(mte.ago(mte.timeUnit.day(6)), 'last week', '6d')
	t.equals(mte.ago(mte.timeUnit.day(13)), 'last week', '13d')
 t.end()
})

tape('weeks', function doTest (t) {
	t.equals(mte.ago(mte.timeUnit.week(2)), '2 weeks ago', '2w')
	t.equals(mte.ago(mte.timeUnit.week(3)), '3 weeks ago', '3w')
 t.end()
})

tape('month', function doTest (t) {
	t.equals(mte.ago(mte.timeUnit.week(4)), 'last month', '4w')
	t.equals(mte.ago(mte.timeUnit.day(30)), 'last month', '30d')
	t.equals(mte.ago(mte.timeUnit.day(50)), 'last month', '50d')
 t.end()
})

tape('months', function doTest (t) {
	t.equals(mte.ago(mte.timeUnit.month(2)), '2 months ago', '2mo')
	t.equals(mte.ago(mte.timeUnit.month(10)), '10 months ago', '10mo')
 t.end()
})

tape('year', function doTest (t) {
	t.equals(mte.ago(mte.timeUnit.day(350)), 'last year', '350d')
	t.equals(mte.ago(mte.timeUnit.day(548)), 'last year', '548d')
 t.end()
})

tape('years', function doTest (t) {
	t.equals(mte.ago(mte.timeUnit.year(2)), '2 years ago', '2y')
	t.equals(mte.ago(mte.timeUnit.year(20)), '20 years ago', '20y')
	t.equals(mte.ago(mte.timeUnit.year(100)), '100 years ago', '100y')
	t.equals(mte.ago(mte.timeUnit.year(100000)), '100000 years ago', '100000y')
 t.end()
})
