'use strict'

const EventEmitter = require('events').EventEmitter

class Unread extends EventEmitter {
  constructor (storage) {
    super()
    this.storage = storage
  }

  read (id) {
    if (!validId(id)) return
    const readItems = this.getRead()
    readItems.add(id)
    return this.setRead(readItems)
  }

  readMany (ids) {
    if (!Array.isArray(ids)) {
      console.warn(new Error(`unread: invalid ids to readMany: ${ids}`), ids)
      return
    }

    ids = ids.filter(validId)
    const readItems = this.getRead()
    const setSrc = []
    Array.prototype.push.apply(setSrc, Array.from(readItems))
    Array.prototype.push.apply(setSrc, Array.from(ids))

    return this.setRead(new Set(setSrc))
  }

  unread (id) {
    if (!validId(id)) return
    const readItems = this.getRead()
    readItems.delete(id)
    return this.setRead(readItems)
  }

  toggleRead (id) {
    if (!validId(id)) return
    const isRead = this.isRead(id)
    isRead ? this.unread(id) : this.read(id)

    return !isRead // return false if now unread
  }

  getRead () {
    return new Set(JSON.parse(this.storage.getItem('read-items') || '[]'))
  }

  setRead (items) {
    setImmediate(() => this.emit('readChanged', items))
    return this.storage.setItem('read-items', JSON.stringify(Array.from(items)))
  }

  isRead (id) {
    if (!validId(id)) return
    return this.getRead().has(id)
  }

  bulkReadCheck () {
    const read = this.getRead()
    return function isRead (id) {
      return read.has(id)
    }
  }
}

function validId (id) {
  if (typeof id !== 'string' || !id.length) {
    console.warn(new Error(`unread: invalid id: ${id}`), id)
    return false
  }
  return true
}

module.exports = Unread
