var test = require('tape')

var Listener = require('../listener')

var Client = require('nsolid-rpcclient')
var concat = require('terminus').concat

test('random port', function (t) {
  var l = new Listener()
  t.ok(l.server, 'provides a server')
  l.server.unref()
  t.ok(l instanceof Listener, 'listener created')
  l.on('listening', function () {
    var address = l.getAddress()
    t.ok(address.port && address.port > 0, 'port is a number > 0')
    t.ok(address.ips && address.ips.length > 0, 'address as a list of ips')
    console.log(address)
    t.end()
  })
})

test('specified port', function (t) {
  var port = 28282
  var l = new Listener(port)
  t.ok(l.server, 'provides a server')
  l.server.unref()
  t.ok(l instanceof Listener, 'listener created')
  l.on('listening', function () {
    var address = l.getAddress()
    t.equal(address.port, port, 'port is the port specified')
    t.ok(address.ips && address.ips.length > 0, 'address as a list of ips')
    console.log(address)
    t.end()
  })
})

test('specified address', function (t) {
  var addr = '127.0.0.1:28283'
  var l = new Listener(addr)
  t.ok(l.server, 'provides a server')
  l.server.unref()
  l.on('listening', function () {
    t.ok(l instanceof Listener, 'listener created')
    var address = l.getAddress()
    t.equal(address.port, 28283, 'port is the port specified')
    t.equal(address.ips.length, 1, 'only one ip is bound')
    t.equal(address.ips[0], '127.0.0.1', 'only 127.0.0.1')
    t.ok(address.ips && address.ips.length > 0, 'address as a list of ips')
    console.log(address)
    t.end()
  })
})

test('127.0.0.1', function (t) {
  var addr = '127.0.0.1'
  var l = new Listener(addr)
  t.ok(l.server, 'provides a server')
  l.server.unref()
  l.on('listening', function () {
    t.ok(l instanceof Listener, 'listener created')
    var address = l.getAddress()
    t.ok(address.port && address.port > 0, 'port is a number > 0')
    t.equal(address.ips.length, 1, 'only one ip is bound')
    t.equal(address.ips[0], '127.0.0.1', 'only 127.0.0.1')
    t.ok(address.ips && address.ips.length > 0, 'address as a list of ips')
    console.log(address)
    t.end()
  })
})

test('127.0.0.1:?', function (t) {
  var addr = '127.0.0.1:?'
  var l = new Listener(addr)
  t.ok(l.server, 'provides a server')
  l.server.unref()
  l.on('listening', function () {
    t.ok(l instanceof Listener, 'listener created')
    var address = l.getAddress()
    t.ok(address.port && address.port > 0, 'port is a number > 0')
    t.equal(address.ips.length, 1, 'only one ip is bound')
    t.equal(address.ips[0], '127.0.0.1', 'only 127.0.0.1')
    t.ok(address.ips && address.ips.length > 0, 'address as a list of ips')
    console.log(address)
    t.end()
  })
})

test('localhost:', function (t) {
  var addr = 'localhost:'
  var l = new Listener(addr)
  t.ok(l.server, 'provides a server')
  l.server.unref()
  l.on('listening', function () {
    t.ok(l instanceof Listener, 'listener created')
    var address = l.getAddress()
    t.ok(address.port && address.port > 0, 'port is a number > 0')
    t.equal(address.ips.length, 1, 'only one ip is bound')
    t.equal(address.ips[0], '127.0.0.1', 'only 127.0.0.1')
    t.ok(address.ips && address.ips.length > 0, 'address as a list of ips')
    console.log(address)
    t.end()
  })
})

test('::', function (t) {
  var addr = '::'
  var l = new Listener(addr)
  t.ok(l.server, 'provides a server')
  l.server.unref()
  l.on('listening', function () {
    t.ok(l instanceof Listener, 'listener created')
    var address = l.getAddress()
    t.ok(address.port && address.port > 0, 'port is a number > 0')
    t.ok(address.ips.length > 1, 'bound multiple interfaces')
    t.ok(address.ips && address.ips.length > 0, 'address as a list of ips')
    console.log(address)
    t.end()
  })
})

test('parse v1', function (t) {
  var l = new Listener()
  l.server.unref()
  var rpc = l.parse({url: '/foo/bar'})
  t.deepEqual(rpc, {command: 'foo', data: null, id: 'bar', app: null, timeout: null}, 'parsed the rpc with id')
  rpc = l.parse({url: '/foo'})
  t.deepEqual(rpc, {command: 'foo', data: null, id: null, app: null, timeout: null}, 'parsed the rpc with no id')
  t.end()
})

test('parse v3', function (t) {
  var l = new Listener()
  l.server.unref()
  var rpc = l.parse({url: '/foo?id=bar'})
  t.deepEqual(rpc, {command: 'foo', data: null, id: 'bar', app: null, timeout: null}, 'parsed the rpc with id')
  rpc = l.parse({url: '/foo?timeout=2000'})
  t.deepEqual(rpc, {command: 'foo', data: null, id: null, app: null, timeout: 2000}, 'parsed the rpc with timeout')
  rpc = l.parse({url: '/foo'})
  t.deepEqual(rpc, {command: 'foo', data: null, id: null, app: null, timeout: null}, 'parsed the rpc with no id')
  rpc = l.parse({url: '/foo?app=baz'})
  t.deepEqual(rpc, {command: 'foo', data: null, id: null, app: 'baz', timeout: null}, 'parsed the rpc with no id with appname')
  rpc = l.parse({url: '/foo/bar?app=baz'})
  t.deepEqual(rpc, {command: 'foo', data: null, id: 'bar', app: 'baz', timeout: null}, 'parsed the v1+v2 rpc with no id with appname')
  rpc = l.parse({url: '/foo?id=bar&app=baz'})
  t.deepEqual(rpc, {command: 'foo', data: null, id: 'bar', app: 'baz', timeout: null}, 'parsed the v1+v2 rpc with no id with appname')
  rpc = l.parse({url: '/foo?id=bar&app=baz&timeout=1000'})
  t.deepEqual(rpc, {command: 'foo', data: null, id: 'bar', app: 'baz', timeout: 1000}, 'parsed the v1+v3 rpc with no id with appname with timeout')
  rpc = l.parse({url: '/foo/'})
  t.deepEqual(rpc, {command: 'foo', data: null, id: null, app: null, timeout: null}, 'trailing slash ignored')
  t.end()
})

test('listener', function (t) {
  var l = new Listener()
  l.server.unref()
  l.on('ping', function (req, rpc, next) {
    t.equal(rpc.command, 'ping', 'correct command')
    t.equal(rpc.data, 'FOO', 'correct data sent')
    next(null, 'PONG')
  })
  l.on('listening', function () {
    var address = 'http://localhost:' + l.getAddress().port
    var c = new Client(address)
    var r = c.send('ping', 'FOO')
    r.pipe(concat(function (body) {
      t.equals(body.toString(), JSON.stringify('PONG'), 'got correct reply')
      t.end()
    }))
  })
})

test('wildcard', function (t) {
  var l = new Listener()
  l.server.unref()
  l.on('*', function (req, rpc, next) {
    t.equal(rpc.command, 'foo', 'correct command via wildcard')
    t.equal(rpc.data, 'BAR', 'correct data sent')
    next(null, 'ZZZ')
  })
  l.on('listening', function () {
    var address = 'http://localhost:' + l.getAddress().port
    var c = new Client(address)
    var r = c.send('foo', 'BAR')
    r.pipe(concat(function (body) {
      t.equals(body.toString(), JSON.stringify('ZZZ'), 'got correct reply')
      t.end()
    }))
  })
})
require('./concat')
